@@ -194,8 +194,18 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         HTTPClient.__init__(self, server_url, token, timeout)
         self.AuthURL = auth_server_url or server_url + "/auth"
         self.MaxConcurrent = max_concurrent_queries
-        self.QueryQueue = None
-        
+        self.AsyncQueue = None
+
+    @property
+    def async_queue(self):
+        if self.AsyncQueue is None:
+            try:    
+                from pythreader import TaskQueue
+            except ModuleNotFoundError:
+                raise ModuleNotFoundError("pythreader module required for asynchronous queries. Use: pip install 'pythreader>=2.7.0'")
+            self.AsyncQueue = TaskQueue(self.MaxConcurrent)
+        return self.AsyncQueue
+
     def resfresh_token(self):
         if self.TokenFile:
              token = open(self.TokenFile, "rb").read()
@@ -210,33 +220,71 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         """
         return self.get_text("data/version")
 
-    def list_datasets(self, namespace_pattern=None, name_pattern=None, with_file_counts=False):
+    def list_datasets(self, namespace_pattern=None, name_pattern=None, with_counts=False):
         """Gets the list of datasets with namespace/name matching the templates. The templates are
         Python ``fnmatch`` module style templates where ``'*'`` matches any substring and ``'?'`` matches a single character.
-        
+
         Parameters
         ----------
         namespace_pattern : str
         name_pattern : str
         with_file_counts : boolean
             controls whether the results should include file counts or dataset names only
-        
+
         Yields
         ------
         generator
             yields dictionaries like {"namespace":..., "name":..., "file_count":...}
         """        
-        url = "data/datasets?with_file_counts=%s" % ("yes" if with_file_counts else "no")
+        #url = "data/datasets?with_file_counts=%s" % ("yes" if with_file_counts else "no")
+        url = "data/datasets?with_counts=no"
         lst = self.get_json(url)
+        promises = []        # [(dataset_dict, promise)]
         for item in lst:
             namespace, name = item["namespace"], item["name"]
             if namespace_pattern is not None and not fnmatch.fnmatch(namespace, namespace_pattern):
                 continue
             if name_pattern is not None and not fnmatch.fnmatch(name, name_pattern):
                 continue
+            if not with_counts:
+                yield item
+            else:
+                # fetch counts asynchronously
+                did = namespace + ":" + name
+                promises.append(self.async_queue.add(self.get_dataset_counts, did, promise_data = item))
+
+        for promise in promises:
+            counts = promise.wait()
+            #print("promise: counts:", counts)
+            item = promise.Data
+            item.update(counts)
             yield item
-    
-    def get_dataset(self, did, namespace=None, name=None):
+
+    def get_dataset_counts(self, did=None, namespace=None, name=None):
+        """Gets single dataset files, subsets, supersets, etc. counts
+        
+        Parameters
+        ----------
+        did : str - "namespace:name"
+        namespace : str
+        name : str
+        
+        Returns
+        -------
+        dict
+            dataset counts or None if the dataset was not found
+        """        
+
+        spec = ObjectSpec(did, namespace=namespace, name=name).did()
+        try:
+            out = self.get_json(f"data/dataset_counts/{spec}")
+            #print("get_dataset_counts", did, " out=", out)
+            return out
+        except NotFoundError:
+            print("get_dataset_counts: None")
+            return None
+
+    def get_dataset(self, did=None, namespace=None, name=None):
         """Gets single dataset
         
         Parameters
@@ -250,9 +298,9 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         dict
             dataset attributes or None if the dataset was not found
         """        
-        
-        if namespace is not None:
-            spec = namespace + ':' + name
+
+        spec = ObjectSpec(did, namespace=namespace, name=name).did()
+
         try:
             return self.get_json(f"data/dataset?dataset={spec}")
         except NotFoundError:
@@ -282,28 +330,34 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
             return None
         
         
-    def create_dataset(self, spec, frozen=False, monotonic=False, creator=None, metadata=None, metadata_requirements=None, description=""):
+    def create_dataset(self, spec, frozen=False, monotonic=False, metadata=None, metadata_requirements=None, 
+            files_query=None, subsets_query=None,
+            description=""):
+
         """Creates new dataset. Requires client authentication.
-        
+
         Parameters
         ----------
         spec : str
             "namespace:name"
         frozen : bool
         monotonic : bool
-        creator : str
-            Dataset creator. Ignored if the user is not an admin
         metadata : dict
             Dataset metadata
         metadata_requirements : dict
             Metadata requirements for files in the dataset
+        file_query : str
+            Run MQL file query and add resulting files to the new dataset
+        dataset_query : str
+            Run MQL dataset query and add resulting datasets to the new dataset as subsets
         description : str
-        
+
         Returns
         -------
         dict
             created dataset attributes
-        """   
+        """
+
         namespace, name = spec.split(":",1)     
         params = {
             "namespace":    namespace,
@@ -311,9 +365,10 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
             "frozen":       frozen,
             "monotonic":    monotonic,
             "metadata":     metadata or {},
-            "metadata_requirements":    metadata_requirements or {},
-            "creator":      creator,
-            "description":  description or ""
+            "metadata_requirements":    metadata_requirements or None,
+            "description":  description or "",
+            "files_query":  files_query or None,
+            "subsets_query":  subsets_query or None
         }
         url = f"data/create_dataset"
         return self.post_json(url, params)
@@ -331,13 +386,15 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         url = f"data/add_child_dataset?parent={parent_spec}&child={child_spec}"
         return self.get_text(url)
         
-    def add_files(self, dataset, file_list, namespace=None):
+    def add_files(self, dataset, file_list=None, namespace=None, query=None):
         """Add existing files to an existing dataset. Requires client authentication.
         
         Parameters
         ----------
         dataset : str
             "namespace:name" or "name", if namespace argument is given
+        query : str
+            MQL query to run and add files matching the query
         file_list : list
             List of dictionaries, one dictionary per file. Each dictionary must contain either a file id
         
@@ -365,7 +422,11 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         -------
         list
             list of dictionaries, one dictionary per file with file ids: { "fid": "..." }
-        """        
+        
+        Notes
+        -----
+        Either ``file_list`` or ``query`` must be specified, but not both
+        """
             
         default_namespace = namespace
         if ':' not in dataset:
@@ -375,12 +436,25 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
 
         url = f"data/add_files?dataset={dataset}"
         
-        data = []
-        for f in file_list:
-            spec = ObjectSpec.from_dict(f, default_namespace)
-            spec.validate()
-            data.append(spec.as_dict())
-        out = self.post_json(url, data)
+        if (file_list is None) == (query is None):
+            raise ValueError("Either file_list or query must be specified, but not both")
+        
+        params = {
+            "namespace": namespace,
+        }
+        if file_list:
+            lst = []
+            for f in file_list:
+                spec = ObjectSpec.from_dict(f, default_namespace)
+                spec.validate()
+                lst.append(spec.as_dict())
+            params["file_list"] = lst
+        elif query:
+            params["query"] = query
+        else:
+            raise ValueError("Either file_list or query must be specified, but not both")
+            
+        out = self.post_json(url, params)
         return out
 
     def declare_file(self, did=None, namespace=None, name=None, auto_name=None,
@@ -854,20 +928,13 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
             See notes below for more on how to use this method.
         """
         
-        if self.QueryQueue is None:
-            try:    
-                from pythreader import TaskQueue
-            except ModuleNotFoundError:
-                raise ModuleNotFoundError("pythreader module required for asynchronous queries. Use: pip install 'pythreader>=2.7.0'")
-            self.QueryQueue = TaskQueue(self.MaxConcurrent)
-            
-        return self.QueryQueue.add_lambda(self.query, query, promise_data=data, **args)
+        return self.async_queue.add_lambda(self.query, query, promise_data=data, **args)
 
     def wait_queries(self):
-        """Wait for all issued asynchronous queries to complete
         """
-        if self.QueryQueue is not None:
-            self.QueryQueue.waitUntilEmpty()
+        Wait for all issued asynchronous queries to complete
+        """
+        self.async_queue.waitUntilEmpty()
     
     def create_namespace(self, name, owner_role=None, description=None):
         """Creates new namespace. Requires client authentication.
